@@ -1,5 +1,5 @@
 /*
- *  Copyright 2007-2020 by the individuals mentioned in the source code history
+ *  Copyright 2007-2021 by the individuals mentioned in the source code history
  *
  *  Licensed under the Apache License, Version 2.0 (the "License");
  *  you may not use this file except in compliance with the License.
@@ -37,6 +37,7 @@
 #include <Eigen/Core>
 #include <Eigen/Dense>
 #include "minicsv.h"
+#include "penalty.h"
 
 struct populateLocation {
 	int from;
@@ -96,6 +97,10 @@ class omxMatrix {
 	unsigned short colMajor;			// used for quick transpose
 	unsigned short hasMatrixNumber;		// is this object in the matrix or algebra arrays?
 	int matrixNumber;					// the offset into the matrices or algebras arrays
+  void setMatrixNumber(int xx) {
+    hasMatrixNumber = 1;
+    matrixNumber = xx;
+  };
 
 	SEXP owner;	// The R object owning data or NULL if we own it.
 
@@ -114,6 +119,7 @@ class omxMatrix {
 	bool canDiscard();
 	omxAlgebra* algebra;				// If it's not an algebra, this is NULL.
 	omxFitFunction* fitFunction;		// If it's not a fit function, this is NULL.
+  std::unique_ptr<Penalty> penalty;
 
 	std::string nameStr;
 	const char *name() const { return nameStr.c_str(); }
@@ -140,6 +146,8 @@ class omxMatrix {
 	void copyAttr(omxMatrix *src);
 	bool isSimple() const { return !algebra && !fitFunction && populate.size()==0; };
 	bool isAlgebra() const { return algebra != 0; }
+	bool isFitFunction() const { return fitFunction != 0; }
+	bool isPenalty() const { return penalty != 0; }
 	int numNonConstElements() const;
 	template <typename T> void loadFromStream(T &st);
 	int size() const { return rows * cols; }
@@ -489,7 +497,7 @@ std::string mxStringifyMatrix(const char *name, const Eigen::DenseBase<T> &mat,
 {
 	std::string buf;
 
-	if (!debug && mat.rows() * mat.cols() > 1000) {
+	if (!debug && mat.rows() * mat.cols() > 1500) {
 		buf = string_snprintf("%s is too large to print # %dx%d\n",
 				name, mat.rows(), mat.cols());
 		return buf;
